#!/usr/bin/python

import time
import pwd
import logging
from logging.handlers import RotatingFileHandler

class tcpNetstat:
    
    PROC_TCP = "/proc/net/tcp"
    STATE = {
            '01':'ESTABLISHED',
            '02':'SYN_SENT',
            '03':'SYN_RECV',
            '04':'FIN_WAIT1',
            '05':'FIN_WAIT2',
            '06':'TIME_WAIT',
            '07':'CLOSE',
            '08':'CLOSE_WAIT',
            '09':'LAST_ACK',
            '0A':'LISTEN',
            '0B':'CLOSING'
            }

    def __init__(self):
        self.content = None
        self.tcpConnectionList = []
        self.logger            = logging.getLogger("tcpNetstat")

    def _load(self):
        ''' Read the table of tcp connections & remove header  '''
        with open(self.PROC_TCP,'r') as f:
            content = f.readlines()
            content.pop(0)
            f.close()
        return content

    def _ip(self,s):
        ip = [str(int(s[6:8],16)),str(int(s[4:6],16)),str(int(s[2:4],16)),str(int(s[0:2],16))]
        return '.'.join(ip)

    def _remove_empty(self,array):
        return [x for x in array if x !='']

    def _convert_ip_port(self, array):
        host,port = array.split(':')
        return self._ip(host),str(int(port,16))

    def _getTcpConnections(self):
        '''
        Function to return a list with status of tcp connections at linux systems
        '''
        result = []
        self.content = self._load()
        for line in self.content:
            line_array = self._remove_empty(line.split(' '))     # Split lines and remove empty spaces.
            l_host,l_port = self._convert_ip_port(line_array[1]) # Convert ipaddress and port from hex to decimal.
            r_host,r_port = self._convert_ip_port(line_array[2]) 
            #tcp_id = line_array[0]
            state = self.STATE[line_array[3]]
            uid = pwd.getpwuid(int(line_array[7]))[0]       # Get user from UID.
            #inode = line_array[9]                           # Need the inode to get process pid.

            nline = [uid, l_host+':'+l_port, r_host+':'+r_port, state]
            result.append(nline)
        return result

    def _compareList(self, netstatList):
        currentElements = []
        found = False
        for curElem in netstatList:
            for oldElem in self.tcpConnectionList:
                if curElem == oldElem:
                    found = True
                    currentElements.append(curElem)
            if not found:
                self.logger.info("Add   : {}".format(curElem))
                currentElements.append(curElem)
            found = False

        if len(currentElements):
            for elem in self.tcpConnectionList:
                if elem not in currentElements:
                    self.logger.info("Remove: {}".format(elem))
        
        self.tcpConnectionList = currentElements


    def processTcpConnections(self):

        result = self._getTcpConnections()

        if len(self.tcpConnectionList) == 0:
            self.tcpConnectionList = result
            for elem in self.tcpConnectionList:
                self.logger.info("Add   : {}".format(elem))
        else:
            self._compareList(result)



if __name__ == '__main__':
    rotHandler = RotatingFileHandler("/var/log/tcpConnection.log", maxBytes=2*1024*1024, backupCount=3)
    logging.basicConfig(filename="/var/log/tcpConnection.log", filemode='w',level=logging.INFO,handlers=[rotHandler],format='%(asctime)s %(name)s %(levelname)s %(message)s')
    l = logging.getLogger("tcpNetStat")
    l.info("tcpNetsat app started")
    tcpConn = tcpNetstat()
    while True:
        tcpConn.processTcpConnections()
        time.sleep(1)